@@ -31,22 +31,37 @@
 #include <stdlib.h>
 #include <string.h>
 
-char * str_dup(const char *str)
+char *str_dup(const char *str)
 {
-  return str ? strcpy (malloc(strlen(str) + 1), str) : NULL;
+    char *dup = NULL;
+
+    if (str) {
+        size_t size = strlen(str) + 1;
+        dup = malloc(size);
+        if (dup) {
+            memcpy(dup, str, size);
+        }
+    }
+    return dup;
 }
 
-char * str_printf(const char *fmt, ...)
+char *str_printf(const char *fmt, ...)
 {
     /* Guess we need no more than 100 bytes. */
-    int len;
     va_list ap;
-    int size = 100;
-    char *tmp, *str = NULL;
+    int     len;
+    int     size = 100;
+    char   *tmp, *str = NULL;
+
+    while (1) {
+
+        tmp = realloc(str, size);
+        if (tmp == NULL) {
+            X_FREE(str);
+            return NULL;
+        }
+        str = tmp;
 
-    str = malloc(size);
-    while (1) 
-    {
         /* Try to print in the allocated space. */
         va_start(ap, fmt);
         len = vsnprintf(str, size, fmt, ap);
@@ -62,16 +77,10 @@ char * str_printf(const char *fmt, ...)
             size = len+1; /* precisely what is needed */
         else           /* glibc 2.0 */
             size *= 2;  /* twice the old size */
-
-        tmp = realloc(str, size);
-        if (tmp == NULL) {
-            return str;
-        }
-        str = tmp;
     }
 }
 
-char *str_next_line(char *p)
+const char *str_next_line(const char *p)
 {
     while (*p && *p != '\r' && *p != '\n') {
         p++;
@@ -83,9 +92,9 @@ char *str_next_line(char *p)
     return p;
 }
 
-char *str_skip_white(char *p)
+const char *str_skip_white(const char *p)
 {
-    while (*p && (*p == '\r' || *p == '\n' || *p == ' ')) {
+    while (*p && (*p == ' ' || *p == '\r' || *p == '\n' || *p == '\t')) {
         p++;
     }
 
